/* la1_5.c */
/* La1,5PCI programming example */

#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <unistd.h>
#include <string.h>
#include <stropts.h>
#include <math.h>
#include <time.h>

#include <sys/select.h>
#include <sys/ioctl.h>
#include <sys/mman.h>

#include "lapci.h"
#include "plx9054.h"

// IOMEM =1 - use IO space, IOMEM = 0 - use memory space
#define IOMEM 1

#define CLOCK   10000000.0
#define CLOCK0  50000000.0

typedef struct  _LAData
{
    int     fChannell;
    int     nChannell;
    int     Gain;
    float   Freq;
    int     BufferSize;
    int     IntEnable;
    int     DiffMode;
} LAData;

/* Open LaPCI device */
LAPCI_HANDLE LaPCIOpen(char *Name, int DevNum)
{
	int i,j,d;
	char devName[12];
	LaPCI_dev_info devInfo;

	j=0;
	for(i=0;i<8;i++)
	{
		sprintf(devName,"/dev/lapci%d",i);
		d = open(devName, O_RDWR);
		if(d==-1)
			continue;
		ioctl(d, LAPCI_IOCTL_GETINFO, &devInfo);
		printf("%08x PLX%x %s\n",devInfo.subsys_id, devInfo.PLXrev, devInfo.name);
		if(strcmp(devInfo.name,Name)!=0)
		{
			close(d);
			continue;
		}
		j++;
		if(j==DevNum)
			return (LAPCI_HANDLE) d;
		close(d);
	}
	return -1;
}
/*======================================================================*/
unsigned long GetPCIValue(LAPCI_HANDLE DevHandle, int reg, int bsize)
{
    LaPCI_IO_struct IOStruct;

	IOStruct.Size=bsize;
	IOStruct.Region=1;
	IOStruct.Address=reg;
	
	if(IOMEM)
		ioctl(DevHandle, LAPCI_IOCTL_INP, &IOStruct);
	else
	{
		IOStruct.Region=0;
		ioctl(DevHandle, LAPCI_IOCTL_INMEM, &IOStruct);
	}

    return IOStruct.Data.lVal;
}
/*----------------------------------------------------------------------*/
int  SetPCIValue(LAPCI_HANDLE DevHandle, int reg, int bsize, unsigned long val)
{
    LaPCI_IO_struct IOStruct;

	IOStruct.Size=bsize;
	IOStruct.Region=1;
	IOStruct.Address=reg;
	IOStruct.Data.lVal=val;

	if(IOMEM)
		return ioctl(DevHandle, LAPCI_IOCTL_OUTP, &IOStruct);
	else
	{
		IOStruct.Region=0;
		return ioctl(DevHandle, LAPCI_IOCTL_OUTMEM, &IOStruct);
	}
}

void StartDMA(LAPCI_HANDLE DevHandle, int dma, unsigned long size)
{
	DMAParameters dp;

	dp.unMode = 0;
    dp.DMAControl.unBusWidth               = 1;
    dp.DMAControl.unWaitStates             = 0;
    dp.DMAControl.unReadyInputEnable       = 1;
    dp.DMAControl.unBTERMInputEnable       = 0;
    dp.DMAControl.unLocalBurstEnable       = 1;
    dp.DMAControl.unChaining               = 0;
    dp.DMAControl.unDoneInterruptEnable    = 0;
    dp.DMAControl.unLocalAddressMode       = 1;
    dp.DMAControl.unDemandMode             = 1;
    dp.DMAControl.unWriteAndInvalidateMode = 0;
    dp.DMAControl.unEOTEnable			   = 0;
    dp.DMAControl.unStopMode               = 0;
    dp.DMAControl.unClearCountMode         = 0;
    dp.DMAControl.unInterruptSelect        = 1;
    dp.DMAControl.unReserved               = 0;
    dp.unLocalAddress                      = 0x00000000L;
    dp.ucFIFOPCI2LocalFull                 = 0;
    dp.ucFIFOLocal2PCIEmpty                = 0;
    dp.ucFIFOLocal2PCIFull                 = 8;
    dp.ucFIFOPCI2LocalEmpty                = 0;
	dp.unSize = size;

	if(dma==0)
	{
		ioctl(DevHandle, LAPCI_IOCTL_STOP_DMA0, 0);
		ioctl(DevHandle, LAPCI_IOCTL_START_DMA0, &dp);
	}

	if(dma==1)
	{
		ioctl(DevHandle, LAPCI_IOCTL_STOP_DMA1, 0);
		ioctl(DevHandle, LAPCI_IOCTL_START_DMA1, &dp);
	}
}

void StopDMA(LAPCI_HANDLE DevHandle, int dma)
{
	if(dma==0)
	{
		ioctl(DevHandle, LAPCI_IOCTL_STOP_DMA0, 0);
	}

	if(dma==1)
	{
		ioctl(DevHandle, LAPCI_IOCTL_STOP_DMA1, 0);
	}
}

int GetReadyDMA(LAPCI_HANDLE DevHandle, int dma)
{
	int ret;

	ret=dma;
	ioctl(DevHandle, LAPCI_IOCTL_DMARDY, &ret);
	return ret;
}
//------------------------------------------------------------------------
void PortIO(LAPCI_HANDLE d, int mode, void* adr)
{
    if(mode==0)
    {
		SetPCIValue(d, 10*4, 1, *(char*)adr);
    }
	else
	{
		*(char*)adr = GetPCIValue(d, 10*4, 1) & 0xFF;
	}
}
/*----------------------------------------------------------------------*/
void  Setup8254(LAPCI_HANDLE d, int timer, int mode, int cnt)
{
    int i;

	mode = ((mode & 7) << 1) | 0x30;
    switch(timer)
    {
		default : timer = 0;    break;
		case 0  :               break;
		case 1  : mode |= 0x40; break;
		case 2  : mode |= 0x80; break;
    }
    for(i=0; i<1000; ++i);
    SetPCIValue(d, 7*4, 1, mode);
    for(i=0; i<1000; ++i);
    SetPCIValue(d, (4+timer)*4, 1, cnt & 0xFF);
    for(i=0; i<1000; ++i);
    SetPCIValue(d, (4+timer)*4, 1, cnt >> 8);
}
/*----------------------------------------------------------------------*/
unsigned short Get8254(LAPCI_HANDLE d, int timer)
{
    int mode;
	unsigned short valLo, valHi;

    switch(timer)
    {
		default : timer = 0;
		case 0  : mode  = 0;    break;
		case 1  : mode  = 0x40; break;
		case 2  : mode  = 0x80; break;
    }
    SetPCIValue(d, 7*4, 1, mode);
    valLo = GetPCIValue(d, (4+timer)*4, 1);
    valHi = GetPCIValue(d, (4+timer)*4, 1);
    return (valHi << 8) | valLo;
}
/*----------------------------------------------------------------------*/
void    GetCLOCK(float freq, float* oclock, int* div)
{
    int i,n;
	double fm, clock,f;

	*div    = (int)(CLOCK0 / CLOCK);
    *oclock = CLOCK0 / (*div);
    fm = 3.4e36;
    for(i=(*div); i<32; ++i)
    {
        clock = CLOCK0 / i;
        n = (int)(clock / freq + 0.5f);
        f = fabs(freq - clock / n);
        if(f < fm)
        {
            fm = f;
            *oclock = (float)(clock);
            *div    = i;
        }
    }
}
/*----------------------------------------------------------------------*/
int     Start(LAPCI_HANDLE d, LAData *la)
{
    int nPort15 = 5;
	int i,nFreq;
    float clock = 10000000.0f;
	short fifo[32];

    GetCLOCK(la->Freq, &clock, &nPort15);
    nFreq = (int)(clock / la->Freq + 0.5f);

    printf("\n From = %d, To = %d, Gain = %d", la->fChannell, la->nChannell, la->Gain);
    printf("\n Freq = %f, Port15 = %d, Cnt = %d\n", la->Freq, nPort15, nFreq);

    SetPCIValue(d, 9*4, 1, 0);                   // set CSR to 0
    SetPCIValue(d,12*4, 1, 0);                   // disable GATE

    SetPCIValue(d,15*4, 1, nPort15-1);           // set freq div between 5..31

    SetPCIValue(d, 3*4, 1, 0);                   // clear FIFO
    SetPCIValue(d, 8*4, 1, 0);
    SetPCIValue(d,14*4, 1, 0);

//    if(la->IntEnable)
//        pla->MaskIRQ();

    SetPCIValue(d, 9*4, 1, 0x02);
    SetPCIValue(d,13*4, 1, 0);
    SetPCIValue(d, 9*4, 1, 0);
    StartDMA(d,0,(la->BufferSize)*sizeof(short));

 //   if(la->IntEnable)
 //       pla->UnMaskIRQ();

    SetPCIValue(d, 2*4, 1, (la->nChannell - 1) | (la->DiffMode ? 0x20 : 0));
    SetPCIValue(d, 16*4, 1, 0);
    
    for(i=0; i<la->nChannell; ++i)
        fifo[i] = (la->Gain << 5) | ((la->fChannell + la->nChannell - 1 - i) & 0x1F);
    for(i=0; i<la->nChannell; ++i)
        SetPCIValue(d, 1*4, 2, fifo[i]&0x1ff);
    SetPCIValue(d, 1*4, 2, 0);

    Setup8254(d, 0, 2, nFreq);
    Setup8254(d, 1, 0, 1);
    Setup8254(d, 2, 0, 1);

    SetPCIValue(d, 9*4, 1, 0x20 | 0x02 | 0x08);
    SetPCIValue(d,12*4, 1, 0x01 | 0x08); // enable GATE, enable OUT

    return 1;
}
/*----------------------------------------------------------------------*/
void  Stop(LAPCI_HANDLE d)
{
 //   pla->MaskIRQ();

    SetPCIValue(d, 12*4, 1, 0);   // disable GATE

    if(GetReadyDMA(d,0) != 0)
    {
        SetPCIValue(d, 9*4, 1, GetPCIValue(d, 9*4, 1) | 0x02);
        SetPCIValue(d, 13*4, 1, 0);
    }
	
    StopDMA(d,0);
    SetPCIValue(d, 9*4, 1, 0);
}
/*======================================================================*/

int main(int argc, char **argv)
{
	LAPCI_HANDLE d;
	int i,fd;
	int DMAsize;
	short *DMAbuff;
	LAData InitParams;
	unsigned char PortData; 
	struct timespec tv;
		
	d = LaPCIOpen("LA1_5PCI",1);
	if(d < 0)
	{
		printf("Can't open La1.5 device\n");
		return d;
	}
	printf("Device La1.5 found\n");
	
	/* Allocate DMA buffer */
	DMAsize = 1024*10;
	DMAbuff = mmap(0, DMAsize*sizeof(short), PROT_READ | PROT_WRITE, MAP_SHARED, d, 0);
	if(DMAbuff == MAP_FAILED)
	{
		puts("Can't map!");
		return -1;
	}

    InitParams.fChannell  = 0;
    InitParams.nChannell  = 1;
    InitParams.Gain       = 0;
    InitParams.Freq       = 10000.0f;
    InitParams.BufferSize = DMAsize;
    InitParams.IntEnable  = 0;
    InitParams.DiffMode   = 0;

	tv.tv_sec=0;
	tv.tv_nsec=2000;

    Start(d, &InitParams);

    for(i=0; i<1000; ++i)
    {
		if(GetReadyDMA(d,0) != 0)
			break;
		nanosleep(&tv, &tv);
    }
    printf("\n %s", i < 1000 ? "Ready\n" : "Not Ready\n");
    Stop(d);

	for(i=0; i<8; i++)
		printf("%8d", DMAbuff[i]);
	printf("\n"); 

	/* Write ADC data to file data1.dat */
	fd=open("data1.dat",O_WRONLY|O_CREAT|O_TRUNC);
	write(fd,DMAbuff,DMAsize*sizeof(short));
	close(fd);

	PortData=0xaa;
	PortIO(d, 0, &PortData);	// Write 8bit port
	PortData=0xff;
	PortIO(d, 1, &PortData);	// Read 8bit port
	printf("Port=%x\n",PortData);

	printf("Done\n");

	munmap(DMAbuff, DMAsize*sizeof(short)); 
	close(d);
	return 0;
}
