/* lapci.h */

#include <linux/ioctl.h> 

#define LAPCI_MAJOR 253
#define LAPCI_SLOT_MASK 0xff

#define PCI_VENDOR_ID_PLX   	0x10b5
#define PCI_DEVICE_ID_PLX_9050	0x9050
#define PCI_DEVICE_ID_PLX_9052	0x9052
#define PCI_DEVICE_ID_PLX_9054	0x9054
#define PCI_DEVICE_ID_PLX_9080	0x9080

/* La48d */
#define SUB_DEVICE_ID_LA48D 0x1
#define SUB_VENDOR_ID_LA48D 0xddd1
#define SUB_SYSTEM_ID_LA48D 0x0001ddd1

/* Lan10M8 family*/
#define SUB_DEVICE_ID_LAn10M8_1 0x0001
#define SUB_SYSTEM_ID_LAn10M8_1 0x0001adc5
#define SUB_DEVICE_ID_LAn10M8_2 0x0101
#define SUB_SYSTEM_ID_LAn10M8_2 0x0101adc5
#define SUB_DEVICE_ID_LAn10M8_3 0x0201
#define SUB_SYSTEM_ID_LAn10M8_3 0x0201adc5
#define SUB_DEVICE_ID_LAn10M8_4 0x0301
#define SUB_SYSTEM_ID_LAn10M8_4 0x0301adc5
#define SUB_DEVICE_ID_LAn10M8_5 0x0401
#define SUB_SYSTEM_ID_LAn10M8_5 0x0401adc5
#define SUB_DEVICE_ID_LAn10M8_6 0x0501
#define SUB_SYSTEM_ID_LAn10M8_6 0x0501adc5
#define SUB_VENDOR_ID_LAn10M8   0xadc5

/* La1.5 family*/
#define SUB_DEVICE_ID_LA1_5_1  0x0002
#define SUB_SYSTEM_ID_LA1_5_1  0x0002adc2
#define SUB_DEVICE_ID_LA1_5_2  0x0102
#define SUB_SYSTEM_ID_LA1_5_2  0x0102adc2
#define SUB_DEVICE_ID_LA1_5_3  0x0202
#define SUB_SYSTEM_ID_LA1_5_3  0x0202adc2
#define SUB_DEVICE_ID_LA1_5_4  0x0302
#define SUB_SYSTEM_ID_LA1_5_4  0x0302adc2
#define SUB_DEVICE_ID_LA1_5_5  0x0502
#define SUB_SYSTEM_ID_LA1_5_5  0x0502adc2
#define SUB_DEVICE_ID_LA1_5_6  0x1002
#define SUB_SYSTEM_ID_LA1_5_6  0x1002adc2
#define SUB_DEVICE_ID_LA1_5_7  0x1102
#define SUB_SYSTEM_ID_LA1_5_7  0x1102adc2
#define SUB_DEVICE_ID_LA1_5_8  0x1202
#define SUB_SYSTEM_ID_LA1_5_8  0x1202adc2
#define SUB_DEVICE_ID_LA1_5_9  0x1302
#define SUB_SYSTEM_ID_LA1_5_9  0x1302adc2
#define SUB_DEVICE_ID_LA1_5_10 0x1502
#define SUB_SYSTEM_ID_LA1_5_10 0x1502adc2
#define SUB_VENDOR_ID_LA1_5    0xadc2

typedef struct _LaPCI_dev_info
{
	int subsys_id;
	char name[16];
	int revision;
	int PLXrev;
} LaPCI_dev_info;

/* Device info table */
LaPCI_dev_info di_La48d={SUB_SYSTEM_ID_LA48D,"LA48D", 1};

LaPCI_dev_info di_Lan10M8_1={SUB_SYSTEM_ID_LAn10M8_1,"LAN10M8PCI",  1};
LaPCI_dev_info di_Lan10M8_2={SUB_SYSTEM_ID_LAn10M8_2,"LAN10M8PCI",  2};
LaPCI_dev_info di_Lan10M8_3={SUB_SYSTEM_ID_LAn10M8_3,"LAN10M8PCI1", 1};
LaPCI_dev_info di_Lan10M8_4={SUB_SYSTEM_ID_LAn10M8_4,"LAN10M8PCI1", 2};
LaPCI_dev_info di_Lan10M8_5={SUB_SYSTEM_ID_LAn10M8_5,"LAN10M8PCI1", 3};
LaPCI_dev_info di_Lan10M8_6={SUB_SYSTEM_ID_LAn10M8_6,"LAN10M8PCI1", 4};

LaPCI_dev_info di_La1_5_1 ={SUB_SYSTEM_ID_LA1_5_1, "LA1_5PCI", 1};
LaPCI_dev_info di_La1_5_2 ={SUB_SYSTEM_ID_LA1_5_2, "LA1_5PCI", 2};
LaPCI_dev_info di_La1_5_3 ={SUB_SYSTEM_ID_LA1_5_3, "LA1_5PCI", 3};
LaPCI_dev_info di_La1_5_4 ={SUB_SYSTEM_ID_LA1_5_4, "LA1_5PCI", 4};
LaPCI_dev_info di_La1_5_5 ={SUB_SYSTEM_ID_LA1_5_5, "LA1_5PCI", 5};
LaPCI_dev_info di_La1_5_6 ={SUB_SYSTEM_ID_LA1_5_6, "LA1_5PCI", 6};
LaPCI_dev_info di_La1_5_7 ={SUB_SYSTEM_ID_LA1_5_7, "LA1_5PCI", 7};
LaPCI_dev_info di_La1_5_8 ={SUB_SYSTEM_ID_LA1_5_8, "LA1_5PCI", 8};
LaPCI_dev_info di_La1_5_9 ={SUB_SYSTEM_ID_LA1_5_9, "LA1_5PCI", 9};
LaPCI_dev_info di_La1_5_10={SUB_SYSTEM_ID_LA1_5_10,"LA1_5PCI", 10};

/* Type for treating 4 byte variables with byte by byte */
typedef union _un_l2cval {
	unsigned long	lVal;
	unsigned char	cVal[4];
}un_l2cval; 
/*----------------------------------------------------------------------*/
typedef struct _LaPCI_IO_struct
{
	unsigned int Region;
	unsigned int Size;
	unsigned int Address;
	un_l2cval Data;		
} LaPCI_IO_struct;
/*----------------------------------------------------------------------*/
typedef struct  _DMAControlReg
{
    unsigned 	unBusWidth                  : 2,
				unWaitStates                : 4,
				unReadyInputEnable          : 1,
				unBTERMInputEnable          : 1,
				unLocalBurstEnable          : 1,
				unChaining                  : 1,
				unDoneInterruptEnable       : 1,
				unLocalAddressMode          : 1,
				unDemandMode                : 1,
				unWriteAndInvalidateMode    : 1,
				unEOTEnable                 : 1,
				unStopMode                  : 1,
				unClearCountMode            : 1,
				unInterruptSelect           : 1,
				unReserved                  :14;
} DMAControlReg;
/*----------------------------------------------------------------------*/
typedef struct  _DMAParameters
{
    unsigned long unMode; // bit 0 = 1-read / 0-write, bit 1 = 1 cycle
    union
    {
		unsigned long   _unControl;
		DMAControlReg   _DMAControl;
    } _cReg;
    unsigned long unLocalAddress;
    unsigned char ucFIFOPCI2LocalFull;
    unsigned char ucFIFOLocal2PCIEmpty;
    unsigned char ucFIFOLocal2PCIFull;
    unsigned char ucFIFOPCI2LocalEmpty;
	unsigned long unSize;
} DMAParameters;

#define DMAControl _cReg._DMAControl
#define unControl  _cReg._unControl

/*----------------------------------------------------------------------*/
struct  DMADescriptor
{
    volatile unsigned long unPCIAddress;
    volatile unsigned long unLocAddress;
    volatile unsigned long unByteCounter;
    volatile unsigned long unNext;
};
/*----------------------------------------------------------------------*/
#ifdef __KERNEL__
typedef struct _BOARD_STRUCT
{
	struct list_head head;		/* List head - must go first */ 
	int slot;					/* Slot number */ 
	struct pci_dev *pcidev;		/* PCI device */ 
	int subsys_id;
	char name[16];
	int revision;
	int PLXrev;
	unsigned int IOBaseAdr[6];	/* I/O base address */ 
	unsigned int IOSize[6];
	unsigned int MemBaseAdr[6];	/* Memory base address */
	void *MemRemapAdr[6];	
	unsigned int MemSize[6];
	int IRQStat;
	void *DMAbuff;
	int DMAsize;
	dma_addr_t BusAddr;
} BOARD_STRUCT;
#endif

typedef int LAPCI_HANDLE;

/* I/O control codes */
#define LAPCI_IOCTL_ENAIRQ		_IOW('r',1,int)
#define LAPCI_IOCTL_DISIRQ		_IO('r',2)
#define LAPCI_IOCTL_OUTP		_IOW('r',3,LaPCI_IO_struct)
#define LAPCI_IOCTL_INP			_IOWR('r',4,LaPCI_IO_struct)
#define LAPCI_IOCTL_OUTMEM		_IOW('r',5,LaPCI_IO_struct)
#define LAPCI_IOCTL_INMEM		_IOWR('r',6,LaPCI_IO_struct)
#define LAPCI_IOCTL_GETINFO		_IOR('r',7,LaPCI_dev_info)
#define LAPCI_IOCTL_GETIRQ		_IOR('r',8,int)
#define LAPCI_IOCTL_START_DMA0	_IOW('r',9, DMAParameters)
#define LAPCI_IOCTL_START_DMA1	_IOW('r',10,DMAParameters)
#define LAPCI_IOCTL_STOP_DMA0	_IO('r',11)
#define LAPCI_IOCTL_STOP_DMA1	_IO('r',12)
#define LAPCI_IOCTL_DMARDY		_IOWR('r',14,int)

 