/* plx9050.h */

#ifndef __PLX9050_H
#define __PLX9050_H

/* Control registers - at PCI_BASE_1 */

#define PLX9050_LAS0RR 0x00		/* Local Address Space 0 Range Register */
#define PLX9050_LAS1RR 0x04		/* Local Address Space 1 Range Register */
#define PLX9050_LAS2RR 0x08		/* Local Address Space 2 Range Register */
#define PLX9050_LAS3RR 0x0C		/* Local Address Space 3 Range Register */
#define PLX9050_EROMRR 0x10		/* Expansion ROM Range Register */
#define PLX9050_LAS0_BASE 0x14  /* Local Address Space 0 Local Base Address */
#define PLX9050_LAS1_BASE 0x18  /* Local Address Space 1 Local Base Address */
#define PLX9050_LAS2_BASE 0x1C  /* Local Address Space 2 Local Base Address */
#define PLX9050_LAS3_BASE 0x20  /* Local Address Space 3 Local Base Address */
#define PLX9050_XROM_BASE 0x24  /* Expansion ROM Local Base Address (Remap) */
#define PLX9050_LAS0BRD 0x28	/* Local Address Space 0 Bus Region Descriptor */
#define PLX9050_LAS1BRD 0x2C	/* Local Address Space 1 Bus Region Descriptor */
#define PLX9050_LAS2BRD 0x30	/* Local Address Space 2 Bus Region Descriptor */
#define PLX9050_LAS3BRD 0x34	/* Local Address Space 3 Bus Region Descriptor */
#define PLX9050_EROMBRD 0x38	/* Expansion ROM Bus Region Descriptor */
#define PLX9050_CS0BASE 0x3C	/* Chip Select 0 Base Address */
#define PLX9050_CS1BASE 0x40	/* Chip Select 1 Base Address */
#define PLX9050_CS2BASE 0x44	/* Chip Select 2 Base Address */
#define PLX9050_CS3BASE 0x48	/* Chip Select 3 Base Address */
#define PLX9050_INTCSR  0x4C	/* Interrupt Control/Status Register */
#define PLX9050_CNTRL   0x50	/* User I/O, PCI Target Response, EEPROM, Initialization Control */

#define PLX_LCR_EXTENT 0x54

/* REG_INTCSR bits */

#define PLX9050_ICS_LI1E 0x00000001  /* Local Interrupt 1 Enable */
#define PLX9050_ICS_LI1P 0x00000002  /* Local Interrupt 1 Polarity */
#define PLX9050_ICS_LI1S 0x00000004  /* Local Interrupt 1 Status */
#define PLX9050_ICS_LI2E 0x00000008  /* Local Interrupt 2 Enable */
#define PLX9050_ICS_LI2P 0x00000010  /* Local Interrupt 2 Polarity */
#define PLX9050_ICS_LI2S 0x00000020  /* Local Interrupt 2 Status */
#define PLX9050_ICS_PIE  0x00000040  /* PCI Interrupt Enable */
#define PLX9050_ICS_SI   0x00000080  /* Software Interrupt */

/* PLX_REG_CNTRL bits */

#define PLX9050_CNTRL_USER0PS 0x00000001  /* User I/O 0 or WAITO# Pin Select */
#define PLX9050_CNTRL_USER0D  0x00000002  /* User I/O 0 Direction */
#define PLX9050_CNTRL_USER0   0x00000004  /* User I/O 0 Data */
#define PLX9050_CNTRL_USER1PS 0x00000008  /* User I/O 1 or LLOCK Pin Select */
#define PLX9050_CNTRL_USER1D  0x00000010  /* User I/O 1 Direction */
#define PLX9050_CNTRL_USER1   0x00000020  /* User I/O 1 Data */
#define PLX9050_CNTRL_USER2PS 0x00000040  /* User I/O 2 or CS2 Pin Select */
#define PLX9050_CNTRL_USER2D  0x00000080  /* User I/O 2 Direction */
#define PLX9050_CNTRL_USER2   0x00000100  /* User I/O 2 Data */
#define PLX9050_CNTRL_USER3PS 0x00000200  /* User I/O 3 or CS3 Pin Select */
#define PLX9050_CNTRL_USER3D  0x00000400  /* User I/O 3 Direction */
#define PLX9050_CNTRL_USER3   0x00000800  /* User I/O 3 Data */
#define PLX9050_CNTRL_PCIBAR  0x00003000  /* PCIBAR Enables */
#define PLX9050_CNTRL_PCIR    0x00004000  /* PCI Read Mode */
#define PLX9050_CNTRL_PCIRWF  0x00008000  /* PCI Read with Write Flush Mode */
#define PLX9050_CNTRL_PCIRNF  0x00010000  /* PCI Read No Flush Mode */
#define PLX9050_CNTRL_PCIRNW  0x00020000  /* PCI Read No Write Mode */
#define PLX9050_CNTRL_PCIW    0x00040000  /* PCI Write Mode */
#define PLX9050_CNTRL_TRDC    0x00780000  /* PCI Target Retry Delay Clocks */
#define PLX9050_CNTRL_DSLE    0x00800000  /* Direct Slave Lock Enable */
#define PLX9050_CNTRL_EESK    0x01000000  /* EEPROM Serial Data Clock */
#define PLX9050_CNTRL_EECS    0x02000000  /* EEPROM Chip Select */
#define PLX9050_CNTRL_EEWD    0x04000000  /* EEPROM Write Data */
#define PLX9050_CNTRL_EERD    0x08000000  /* EEPROM Read Data */
#define PLX9050_CNTRL_EECHK   0x10000000  /* EEPROM Present */
#define PLX9050_CNTRL_RCR     0x20000000  /* EEPROM Reload */
#define PLX9050_CNTRL_RESET   0x40000000  /* PCI Adapter Software Reset */

#endif

