/* plx9054.h */

#ifndef __PLX9054_H
#define __PLX9054_H

/* Control registers - at PCI_BASE_1 */

#define PLX9054_LAS0RR		0x00  /* Local Address Space 0 Range Register */
#define PLX9054_LAS0_BASE	0x04  /* Local Address Space 0 Local Base Address */
#define PLX9054_MARBR		0x08  /* Mode/DMA Arbitration */
#define PLX9054_BIGEND		0x0c  /* Big/Little endian descriptor */
#define PLX9054_LMSIC		0x0d  /* Local miscellaneous control */
#define PLX9054_PROT_AREA	0x0e  /* Serial EEPROM write-protected adress boundary */
#define PLX9054_EROMRR		0x10  /* Expansion ROM Range Register */
#define PLX9054_XROM_BASE	0x24  /* Expansion ROM Local Base Address (Remap) */
#define PLX9054_EROMBRD		0x18  /* Expansion ROM Bus Region Descriptor */
#define PLX9054_DMRR		0x1c  /* Local range register for PCI Initiatir-to-PCI*/
#define PLX9054_DMLBAM		0x20  /* Local Bus Base Address Register for PCI Initiator-to-PCI Memory */
#define PLX9054_DMLBAI      0x24  /* Local Base Address Register for PCI Initiator-to-PCI IO config */
#define PLX9054_DMPBAM		0x28  /* PCI Base Address Register for PCI Initiator-to-PCI Memory */
#define PLX9054_DMCFGA      0x2c  /* PCI Configuration Address Register for PCI Initiator-to-PCI IO config*/
#define PLX9054_LAS1RR      0xf0  /* Local Address Space 1 Range Register */
#define PLX9054_LAS1_BASE   0xf4  /* Local Address Space 1 Local Base Address */
#define PLX9054_LAS1BRD     0xf8  /* Local Address Space 1 Bus Region Descriptor */
#define PLX9054_DMDAC       0xfc  /* PCI Initiator PCI Dual address cycle */
#define PLX9054_MAILBOX2    0x48  /* Mailbox register 2 */
#define PLX9054_MAILBOX3    0x4c  /* Mailbox register 3 */
#define PLX9054_MAILBOX4    0x50  /* Mailbox register 4 */
#define PLX9054_MAILBOX5    0x54  /* Mailbox register 5 */
#define PLX9054_MAILBOX6    0x58  /* Mailbox register 6 */
#define PLX9054_MAILBOX7    0x5C  /* Mailbox register 7 */
#define PLX9054_P2LDBELL    0x60  /* PCI-to-Local Doorbell */
#define PLX9054_L2PDBELL    0x64  /* Local-to-PCI Doorbell */
#define PLX9054_INTCSR      0x68  /* Interrupt Control/Status Register */
#define PLX9054_CNTRL       0x6c  /* User I/O, PCI Target Response, EEPROM, Initialization Control */
#define PLX9054_CONFIG_ID   0x70  /* PCI hardcoded configuration ID */
#define PLX9054_REVISION    0x74  /* PCI hardcoded revision ID */

/* DMA Registers */
#define PLX9054_DMAMODE0    0x80  /* DMA Channel 0 Mode */
#define PLX9054_DMAPADR0    0x84  /* DMA Channel 0 PCI Address */
#define PLX9054_DMALADR0    0x88  /* DMA Channel 0 Local Address */
#define PLX9054_DMASIZ0     0x8c  /* DMA Channel 0 Transfer Size (Bytes) */
#define PLX9054_DMADPR0     0x90  /* DMA Channel 0 Descriptor Pointer */
#define PLX9054_DMAMODE1    0x94  /* DMA Channel 1 Mode */
#define PLX9054_DMAPADR1    0x98  /* DMA Channel 1 PCI Address */
#define PLX9054_DMALADR1    0x9c  /* DMA Channel 1 Local Address */
#define PLX9054_DMASIZ1     0xa0  /* DMA Channel 1 Transfer Size (Bytes) */
#define PLX9054_DMADPR1     0xa4  /* DMA Channel 1 Descriptor Pointer */
#define PLX9054_DMACSR0     0xa8  /* DMA Channel 0 Command/Status */
#define PLX9054_DMACSR1     0xa9  /* DMA Channel 1 Command/Status*/
#define PLX9054_DMAARB      0xac  /* DMA Arbitration */
#define PLX9054_DMATHR      0xb0  /* DMA Threshold */
#define PLX9054_DMADAC0     0xb4  /* DMA Channel 0 PCI Dual Address Cycle Address Register */
#define PLX9054_DMADAC1     0xb8  /* DMA Channel 1 PCI Dual Address Cycle Address Register */ 

/* REG_INTCSR bits */
#define PLX9054_ICS_PIE  0x00000100  /* PCI Interrupt Enable */
#define PLX9054_ICS_DBIE 0x00000200  /* Doorbell Interrupt Enable */
#define PLX9054_ICS_ABIE 0x00000400	 /* PCI Abort Interrupt Enable */
#define PLX9054_ICS_LIE  0x00000800  /* Local Interrupt Enable */
#define PLX9054_ICS_RAE  0x00001000  /* Retry Abort Enable */
#define PLX9054_ICS_DBIS 0x00002000  /* Doorbell Interrupt Status */
#define PLX9054_ICS_ABIS 0x00004000	 /* PCI Abort Interrupt Status */
#define PLX9054_ICS_LIS  0x00008000  /* Local Interrupt Status */

/* PLX_REG_CNTRL bits */
#define PLX9054_CNTRL_GPO     0x00010000  /* General Purpose Output */
#define PLX9054_CNTRL_GPI     0x00020000  /* General Purpose Input */
#define PLX9054_CNTRL_USRI    0x00040000  /* User Input */
#define PLX9054_CNTRL_USRO	  0x00080000  /* User Output */
#define PLX9054_CNTRL_EESK    0x01000000  /* EEPROM Serial Data Clock */
#define PLX9054_CNTRL_EECS    0x02000000  /* EEPROM Chip Select */
#define PLX9054_CNTRL_EEWD    0x04000000  /* EEPROM Write Data */
#define PLX9054_CNTRL_EERD    0x08000000  /* EEPROM Read Data */
#define PLX9054_CNTRL_EECHK   0x10000000  /* EEPROM Present */
#define PLX9054_CNTRL_RCR     0x20000000  /* EEPROM Reload */
#define PLX9054_CNTRL_RESET   0x40000000  /* PCI Adapter Software Reset */

#endif

