/* lapci.c */
/* Rudnev - Shilaev PCI ADC/DAC boards driver for Linux */
/* Written by Chepelev Vladimir */
/* Version 1.0.1 (12.04.2005) */ 

#define MODULE 
#define __KERNEL__ 

#include <linux/config.h>
#include <linux/version.h> 
#include <linux/module.h>
#include <linux/init.h>   
#include <linux/kernel.h>
#include <linux/pci.h>
#include <linux/interrupt.h>
#include <linux/ioport.h>
#include <linux/list.h>
#include <linux/pagemap.h>
#include <linux/mm.h>
#include <linux/slab.h>
#include <linux/wrapper.h> 

#include <asm/uaccess.h>
#include <asm/io.h>
#include <asm/delay.h>
#include <asm/page.h>

#include "include/lapci.h"
#include "include/plx9050.h"
#include "include/plx9054.h"

MODULE_DESCRIPTION("Rudnev - Shilaev PCI ADC/DAC driver");
MODULE_AUTHOR("Chepelev Vladimir");
MODULE_LICENSE("GPL"); 
MODULE_PARM(slot, "i");
MODULE_PARM_DESC(slot, "Slot number");
MODULE_PARM(Major, "i");
MODULE_PARM_DESC(Major, "LaPCI major number");

static char device_name[] = "LaPCI";
LIST_HEAD(dev_list);

static struct pci_device_id id_table[] = 
{
	{PCI_VENDOR_ID_PLX, PCI_DEVICE_ID_PLX_9050,
		SUB_VENDOR_ID_LA48D, SUB_DEVICE_ID_LA48D, 0, 0, (long)(&di_La48d)},

	{PCI_VENDOR_ID_PLX, PCI_DEVICE_ID_PLX_9054,
		SUB_VENDOR_ID_LAn10M8, SUB_DEVICE_ID_LAn10M8_1, 0, 0, (long)(&di_Lan10M8_1)},
	{PCI_VENDOR_ID_PLX, PCI_DEVICE_ID_PLX_9054,
		SUB_VENDOR_ID_LAn10M8, SUB_DEVICE_ID_LAn10M8_2, 0, 0, (long)(&di_Lan10M8_2)},
	{PCI_VENDOR_ID_PLX, PCI_DEVICE_ID_PLX_9054,
		SUB_VENDOR_ID_LAn10M8, SUB_DEVICE_ID_LAn10M8_3, 0, 0, (long)(&di_Lan10M8_3)},
	{PCI_VENDOR_ID_PLX, PCI_DEVICE_ID_PLX_9054,
		SUB_VENDOR_ID_LAn10M8, SUB_DEVICE_ID_LAn10M8_4, 0, 0, (long)(&di_Lan10M8_4)},
	{PCI_VENDOR_ID_PLX, PCI_DEVICE_ID_PLX_9054,
		SUB_VENDOR_ID_LAn10M8, SUB_DEVICE_ID_LAn10M8_5, 0, 0, (long)(&di_Lan10M8_5)},
	{PCI_VENDOR_ID_PLX, PCI_DEVICE_ID_PLX_9054,
		SUB_VENDOR_ID_LAn10M8, SUB_DEVICE_ID_LAn10M8_6, 0, 0, (long)(&di_Lan10M8_6)},

	{PCI_VENDOR_ID_PLX, PCI_DEVICE_ID_PLX_9054,
		SUB_VENDOR_ID_LA1_5, SUB_DEVICE_ID_LA1_5_1, 0, 0, (long)(&di_La1_5_1)},
	{PCI_VENDOR_ID_PLX, PCI_DEVICE_ID_PLX_9054,
		SUB_VENDOR_ID_LA1_5, SUB_DEVICE_ID_LA1_5_2, 0, 0, (long)(&di_La1_5_2)},
	{PCI_VENDOR_ID_PLX, PCI_DEVICE_ID_PLX_9054,
		SUB_VENDOR_ID_LA1_5, SUB_DEVICE_ID_LA1_5_3, 0, 0, (long)(&di_La1_5_3)},
	{PCI_VENDOR_ID_PLX, PCI_DEVICE_ID_PLX_9054,
		SUB_VENDOR_ID_LA1_5, SUB_DEVICE_ID_LA1_5_4, 0, 0, (long)(&di_La1_5_4)},
	{PCI_VENDOR_ID_PLX, PCI_DEVICE_ID_PLX_9054,
		SUB_VENDOR_ID_LA1_5, SUB_DEVICE_ID_LA1_5_5, 0, 0, (long)(&di_La1_5_5)},
	{PCI_VENDOR_ID_PLX, PCI_DEVICE_ID_PLX_9054,
		SUB_VENDOR_ID_LA1_5, SUB_DEVICE_ID_LA1_5_6, 0, 0, (long)(&di_La1_5_6)},
	{PCI_VENDOR_ID_PLX, PCI_DEVICE_ID_PLX_9054,
		SUB_VENDOR_ID_LA1_5, SUB_DEVICE_ID_LA1_5_7, 0, 0, (long)(&di_La1_5_7)},
	{PCI_VENDOR_ID_PLX, PCI_DEVICE_ID_PLX_9054,
		SUB_VENDOR_ID_LA1_5, SUB_DEVICE_ID_LA1_5_8, 0, 0, (long)(&di_La1_5_8)},
	{PCI_VENDOR_ID_PLX, PCI_DEVICE_ID_PLX_9054,
		SUB_VENDOR_ID_LA1_5, SUB_DEVICE_ID_LA1_5_9, 0, 0, (long)(&di_La1_5_9)},
	{PCI_VENDOR_ID_PLX, PCI_DEVICE_ID_PLX_9054,
		SUB_VENDOR_ID_LA1_5, SUB_DEVICE_ID_LA1_5_10,0, 0, (long)(&di_La1_5_10)},

	{PCI_VENDOR_ID_PLX, PCI_DEVICE_ID_PLX_9080,
		SUB_VENDOR_ID_LA1_5, SUB_DEVICE_ID_LA1_5_1, 0, 0, (long)(&di_La1_5_1)},
	{PCI_VENDOR_ID_PLX, PCI_DEVICE_ID_PLX_9080,
		SUB_VENDOR_ID_LA1_5, SUB_DEVICE_ID_LA1_5_2, 0, 0, (long)(&di_La1_5_2)},
	{PCI_VENDOR_ID_PLX, PCI_DEVICE_ID_PLX_9080,
		SUB_VENDOR_ID_LA1_5, SUB_DEVICE_ID_LA1_5_3, 0, 0, (long)(&di_La1_5_3)},
	{PCI_VENDOR_ID_PLX, PCI_DEVICE_ID_PLX_9080,
		SUB_VENDOR_ID_LA1_5, SUB_DEVICE_ID_LA1_5_4, 0, 0, (long)(&di_La1_5_4)},
	{PCI_VENDOR_ID_PLX, PCI_DEVICE_ID_PLX_9080,
		SUB_VENDOR_ID_LA1_5, SUB_DEVICE_ID_LA1_5_5, 0, 0, (long)(&di_La1_5_5)},
	{PCI_VENDOR_ID_PLX, PCI_DEVICE_ID_PLX_9080,
		SUB_VENDOR_ID_LA1_5, SUB_DEVICE_ID_LA1_5_6, 0, 0, (long)(&di_La1_5_6)},
	{PCI_VENDOR_ID_PLX, PCI_DEVICE_ID_PLX_9080,
		SUB_VENDOR_ID_LA1_5, SUB_DEVICE_ID_LA1_5_7, 0, 0, (long)(&di_La1_5_7)},
	{PCI_VENDOR_ID_PLX, PCI_DEVICE_ID_PLX_9080,
		SUB_VENDOR_ID_LA1_5, SUB_DEVICE_ID_LA1_5_8, 0, 0, (long)(&di_La1_5_8)},
	{PCI_VENDOR_ID_PLX, PCI_DEVICE_ID_PLX_9080,
		SUB_VENDOR_ID_LA1_5, SUB_DEVICE_ID_LA1_5_9, 0, 0, (long)(&di_La1_5_9)},
	{PCI_VENDOR_ID_PLX, PCI_DEVICE_ID_PLX_9080,
		SUB_VENDOR_ID_LA1_5, SUB_DEVICE_ID_LA1_5_10,0, 0, (long)(&di_La1_5_10)},

	{0, 0, 0, 0, 0, 0, 0}
};

MODULE_DEVICE_TABLE(pci, id_table); 

static int Boards=0;	/* Number of boards found */
static int slot = 0;	/* Slot number */ 
static int flash_delay=10;
static int Major=LAPCI_MAJOR;	/*     */

/* Reset PLX */
static int plx_reset(BOARD_STRUCT *b)
{
	int x;

	if(!b) 
		return -1;
	switch(b->PLXrev)
	{
		case PCI_DEVICE_ID_PLX_9050:
		case PCI_DEVICE_ID_PLX_9052:
			x = inl(b->IOBaseAdr[0] + PLX9050_CNTRL);
			x |= PLX9050_CNTRL_RESET;
			outl(x, b->IOBaseAdr[0] + PLX9050_CNTRL);
			udelay(flash_delay);
			x &= ~PLX9050_CNTRL_RESET;
			outl(x, b->IOBaseAdr[0] + PLX9050_CNTRL);
			udelay(flash_delay);
			break;
		case PCI_DEVICE_ID_PLX_9054:
		case PCI_DEVICE_ID_PLX_9080:
			x = inl(b->IOBaseAdr[0] + PLX9054_CNTRL);
			x |= PLX9054_CNTRL_RESET;
			outl(x, b->IOBaseAdr[0] + PLX9054_CNTRL);
			udelay(flash_delay);
			x &= ~PLX9054_CNTRL_RESET;
			outl(x, b->IOBaseAdr[0] + PLX9054_CNTRL);
			udelay(flash_delay);
			break;

	}
	return 0;
}

/* Enable/disable interrupt from PLX */
static int plx_int_enable(BOARD_STRUCT *b, int State)
{
	int x;

	if(!b) 
		return -1;
	switch(b->PLXrev)
	{
		case PCI_DEVICE_ID_PLX_9050:
		case PCI_DEVICE_ID_PLX_9052:
			x = inl((b->IOBaseAdr[0]) + PLX9050_INTCSR);
			x = x & (~(PLX9050_ICS_PIE|PLX9050_ICS_LI1E|PLX9050_ICS_LI2E));
			if(State & 1) 
				x |= PLX9050_ICS_PIE | PLX9050_ICS_LI1E;
			if(State & 2)
				x |= PLX9050_ICS_PIE | PLX9050_ICS_LI2E;
			outl(x, b->IOBaseAdr[0] + PLX9050_INTCSR);
			break;
		case PCI_DEVICE_ID_PLX_9054:
		case PCI_DEVICE_ID_PLX_9080:
			x = inl((b->IOBaseAdr[0]) + PLX9054_INTCSR);
			x = x & (~(PLX9054_ICS_PIE|PLX9054_ICS_LIE));
			if(State) 
				x |= PLX9054_ICS_PIE | PLX9054_ICS_LIE;
			outl(x, b->IOBaseAdr[0] + PLX9054_INTCSR);
			break;
	}
	return 0;
}

/* Start DMA */
void StartDMA(BOARD_STRUCT *b, int dma, DMAParameters* dmaPar)
{
	u32 fifo,flags,padr;

	fifo = ((u32)(dmaPar->ucFIFOPCI2LocalFull  & 0x0F) <<  0) |
           ((u32)(dmaPar->ucFIFOLocal2PCIEmpty & 0x0F) <<  4) |
           ((u32)(dmaPar->ucFIFOLocal2PCIFull  & 0x0F) <<  8) |
           ((u32)(dmaPar->ucFIFOPCI2LocalEmpty & 0x0F) << 12) |
           ((u32)(dmaPar->ucFIFOPCI2LocalFull  & 0x0F) << 16) |
           ((u32)(dmaPar->ucFIFOLocal2PCIEmpty & 0x0F) << 20) |
           ((u32)(dmaPar->ucFIFOLocal2PCIFull  & 0x0F) << 24) |
           ((u32)(dmaPar->ucFIFOPCI2LocalEmpty & 0x0F) << 28);
    outl(fifo, (b->IOBaseAdr[0]) + PLX9054_DMATHR); 
	outb(0,(b->IOBaseAdr[0]) + (dma == 0 ? PLX9054_DMACSR0  : PLX9054_DMACSR1));	// Disable DMA

	flags = 1L + (dmaPar->unMode & 0x01 ? 0L : 8L);
	padr=0;
	outl(dmaPar->unControl,(b->IOBaseAdr[0]) + (dma == 0 ? PLX9054_DMAMODE0 : PLX9054_DMAMODE1));   // Set DMA Mode
	if((dmaPar->DMAControl).unChaining == 0)
	{	// Linear DMA mode
		outl(b->BusAddr,(b->IOBaseAdr[0]) + (dma == 0 ? PLX9054_DMAPADR0 : PLX9054_DMAPADR1));			// PCI address
		outl(dmaPar->unLocalAddress,(b->IOBaseAdr[0]) + (dma == 0 ? PLX9054_DMALADR0:PLX9054_DMALADR1));// Local address
		outl(dmaPar->unSize,(b->IOBaseAdr[0]) + (dma == 0 ? PLX9054_DMASIZ0  : PLX9054_DMASIZ1));		// DMA size	
	}
	else
	{	// DMA descriptor mode

	}

	outl(flags | padr,(b->IOBaseAdr[0]) + (dma == 0 ? PLX9054_DMADPR0  : PLX9054_DMADPR1));
	outb(1,(b->IOBaseAdr[0]) + (dma == 0 ? PLX9054_DMACSR0  : PLX9054_DMACSR1));	// Enable DMA
	outb(3,(b->IOBaseAdr[0]) + (dma == 0 ? PLX9054_DMACSR0  : PLX9054_DMACSR1));	// Go!	
}

/* Stop DMA */
void StopDMA(BOARD_STRUCT *b, int dma)
{
	int x;

	x = inb((b->IOBaseAdr[0]) + (dma == 0 ? PLX9054_DMACSR0  : PLX9054_DMACSR1));
	if(x & 0x01 && (x & 0x10) == 0)		// Started & Enabled
	{                                  				
		x=inb((b->IOBaseAdr[0]) + PLX9054_REVISION);
		if((b->PLXrev==PCI_DEVICE_ID_PLX_9054)&&(x <= 0x0A))
		{
			outb(0,(b->IOBaseAdr[0]) + (dma == 0 ? PLX9054_DMACSR0  : PLX9054_DMACSR1)); // Disable DMA
		}
		else
		{
			outb(0,(b->IOBaseAdr[0]) + (dma == 0 ? PLX9054_DMACSR0  : PLX9054_DMACSR1)); // Disable DMA
			outb(4,(b->IOBaseAdr[0]) + (dma == 0 ? PLX9054_DMACSR0  : PLX9054_DMACSR1)); // Abort DMA
			outl(0,(b->IOBaseAdr[0]) + (dma == 0 ? PLX9054_DMAMODE0 : PLX9054_DMAMODE1));
		}
	}
}

/* Get DMA status ready */
static int GetReadyDMA(BOARD_STRUCT *b, int dma)
{
	int st, ret;

	if(dma == 0)
    {
 		st=inb((b->IOBaseAdr[0]) + PLX9054_DMACSR0);
		st=st&0xff;
        ret = st & 0x10 ? 1 : 0;
        if((st & 0x01) && (st & 0x10))
            outb(0, (b->IOBaseAdr[0]) + PLX9054_DMACSR0); // Clear enable
        return ret;
    }
    if(dma == 1)
    {
        st=inb((b->IOBaseAdr[0]) + PLX9054_DMACSR1);
		st=st&0xff;
        ret = st & 0x10 ? 1 : 0;
        if((st & 0x01) && (st & 0x10))
            outb(0, (b->IOBaseAdr[0]) + PLX9054_DMACSR1); // Clear enable
        return ret;
    }
    return 0;
}

/* Allocate DMA buffer */
void *alloc_dmabuf(int size, int mask)
{
	unsigned long start, end, p;
	int order, xsize;

	order = 0; xsize = PAGE_SIZE;
	while(xsize<size)
	{
		xsize <<= 1;
		order++;
	}

	start = __get_free_pages(mask, order);
	if(start == 0) return NULL;
	end = start + xsize;
	for(p = start; p < end; p += PAGE_SIZE)
	{
		mem_map_reserve(virt_to_page(p));
	}
	return (void*) start;
}

/* Free DMA buffer */
int free_dmabuf(void *buf, int size)
{
	unsigned long start, end, p;
	int order, xsize;

	if(buf==NULL) return -1;

	order = 0; xsize = PAGE_SIZE;
	while(xsize<size)
	{
		xsize <<= 1;
		order++;
	}

	start = (unsigned long) buf;
	end = start + xsize;

	for(p = start; p < end; p += PAGE_SIZE)
	{
		mem_map_unreserve(virt_to_page(p));
	}
	free_pages(start, order);
	return 0;
} 

/* Open device */
static int device_open(struct inode *inode, struct file *file)
{
	struct list_head *lh;
	BOARD_STRUCT *b;
	int x;

	x = MINOR(inode->i_rdev) & LAPCI_SLOT_MASK;
	b = NULL;
	for(lh = dev_list.next; lh != &dev_list; lh = lh->next)
	{
		b = (BOARD_STRUCT*) lh;
		if(b == NULL) 
			continue;
		if(b->slot == x) 
			break;
	}
	if(b == NULL) 
		return -ENODEV;
	if(b->slot != x)
		return -ENODEV;

	file->private_data=(void*) b;
	MOD_INC_USE_COUNT;
	return 0;
}

/* Close device */
static int device_release(struct inode *inode, struct file *file)
{
	BOARD_STRUCT *b;

	b = (BOARD_STRUCT*) file->private_data;
	if(b == NULL) 
		return -1;
 
	plx_int_enable(b, 0);
	MOD_DEC_USE_COUNT;
	return 0;
}

/* Read from device */
static ssize_t device_read(struct file *file, char *buffer, size_t length, loff_t *offset)
{
	return -EINVAL;
}

/* Write to device */
static ssize_t device_write(struct file *file, const char *buffer, size_t length, loff_t *offset)
{
	return -EINVAL;
}

/* Map DMA buffer */
static int device_mmap(struct file *file, struct vm_area_struct *vma)
{
	BOARD_STRUCT *b;
	unsigned long addr;
	int size;

	b = (BOARD_STRUCT*) file->private_data;
	if(b == NULL) 
		return -1;

	if(b->PLXrev==PCI_DEVICE_ID_PLX_9050)
		return -1;
	if(b->PLXrev==PCI_DEVICE_ID_PLX_9052)
		return -1;

	size = vma->vm_end - vma->vm_start;
	if(b->DMAbuff!=NULL)
		free_dmabuf(b->DMAbuff,b->DMAsize);

	b->DMAbuff=alloc_dmabuf(size,GFP_USER);
	b->BusAddr = virt_to_bus(b->DMAbuff);

	if(b->DMAbuff==NULL)
		return -ENOMEM;
	memset(b->DMAbuff,0,size);

 	addr = virt_to_phys(b->DMAbuff);
	b->DMAsize=size;

	if(remap_page_range(vma->vm_start, addr, size, vma->vm_page_prot)) 
		return -EAGAIN;
	return 0;
}

/* Device I/O control */
static int device_ioctl(struct inode *inode, struct file *file, unsigned ioctl_num, unsigned long param)
{
	BOARD_STRUCT *b;
	LaPCI_dev_info devInfo;
	LaPCI_IO_struct IOStruct;
	DMAParameters dp;
	int bn,x;
	int dma;
 
	b = (BOARD_STRUCT*) file->private_data;
	if(!b) 
		return -1;
	bn = MINOR(inode->i_rdev);

	switch(ioctl_num)
	{
		case LAPCI_IOCTL_ENAIRQ:
			if(copy_from_user(&x, (void*) param, sizeof(int))) return -EFAULT; 
			b->IRQStat=0;
			plx_int_enable(b, x);
			break;
		case LAPCI_IOCTL_DISIRQ:
			plx_int_enable(b, 0);
			break;
		case LAPCI_IOCTL_OUTP:
			if(copy_from_user(&IOStruct, (void*) param, sizeof(LaPCI_IO_struct))) return -EFAULT; 
			switch(IOStruct.Size)
			{
				case 1:
					if((b->IOSize[IOStruct.Region])>0)
						outb((unsigned char)(IOStruct.Data.lVal),(b->IOBaseAdr[IOStruct.Region]) + IOStruct.Address);
					break;
				case 2:
					if((b->IOSize[IOStruct.Region])>0)
						outw((unsigned short)(IOStruct.Data.lVal),(b->IOBaseAdr[IOStruct.Region]) + IOStruct.Address);
					break;
				case 4:
					if((b->IOSize[IOStruct.Region])>0)
						outl(IOStruct.Data.lVal,(b->IOBaseAdr[IOStruct.Region]) + IOStruct.Address);
					break;
			}
			break;
		case LAPCI_IOCTL_INP:
			if(copy_from_user(&IOStruct, (void*) param, sizeof(LaPCI_IO_struct))) return -EFAULT; 
			switch(IOStruct.Size)
			{
				case 1:
					if((b->IOSize[IOStruct.Region])>0)
						IOStruct.Data.lVal=(unsigned long)inb((b->IOBaseAdr[IOStruct.Region]) + IOStruct.Address);
					break;
				case 2:
					if((b->IOSize[IOStruct.Region])>0)
						IOStruct.Data.lVal=(unsigned long)inw((b->IOBaseAdr[IOStruct.Region]) + IOStruct.Address);
					break;
				case 4:
					if((b->IOSize[IOStruct.Region])>0)
						IOStruct.Data.lVal=(unsigned long)inl((b->IOBaseAdr[IOStruct.Region]) + IOStruct.Address);
					break;
			}
			return copy_to_user((void*) param, &IOStruct, sizeof(LaPCI_IO_struct));
			break;
		case LAPCI_IOCTL_OUTMEM:
			if(copy_from_user(&IOStruct, (void*) param, sizeof(LaPCI_IO_struct))) return -EFAULT;
			switch(IOStruct.Size)
			{
				case 1:
					if((b->MemSize[IOStruct.Region])>0)
						writeb((unsigned char)(IOStruct.Data.lVal),(b->MemRemapAdr[IOStruct.Region]) + IOStruct.Address);
					break;
				case 2:
					if((b->MemSize[IOStruct.Region])>0)
						writew((unsigned short)(IOStruct.Data.lVal),(b->MemRemapAdr[IOStruct.Region]) + IOStruct.Address);
					break;
				case 4:
					if((b->MemSize[IOStruct.Region])>0)
						writel(IOStruct.Data.lVal,(b->MemRemapAdr[IOStruct.Region]) + IOStruct.Address);
					break;
			}
			break;
		case LAPCI_IOCTL_INMEM:
			if(copy_from_user(&IOStruct, (void*) param, sizeof(LaPCI_IO_struct))) return -EFAULT;
			switch(IOStruct.Size)
			{
				case 1:
					if((b->MemSize[IOStruct.Region])>0)
						IOStruct.Data.lVal=(unsigned long)readb((b->MemRemapAdr[IOStruct.Region]) + IOStruct.Address);
					break;
				case 2:
					if((b->MemSize[IOStruct.Region])>0)
						IOStruct.Data.lVal=(unsigned long)readw((b->MemRemapAdr[IOStruct.Region]) + IOStruct.Address);
					break;
				case 4:
					if((b->MemSize[IOStruct.Region])>0)
						IOStruct.Data.lVal=(unsigned long)readl((b->MemRemapAdr[IOStruct.Region]) + IOStruct.Address);
					break;
			}
			return copy_to_user((void*) param, &IOStruct, sizeof(LaPCI_IO_struct));
			break;
		case LAPCI_IOCTL_GETINFO:  /* Get board info */
			devInfo.subsys_id = b->subsys_id;
			strcpy(devInfo.name, b->name);
			devInfo.revision = b->revision;
			devInfo.PLXrev = b->PLXrev;
			return copy_to_user((void*) param, &devInfo, sizeof(LaPCI_dev_info));
			break;
		case LAPCI_IOCTL_GETIRQ:  /* Get IRQ status */
			copy_to_user((void*) param, &(b->IRQStat), sizeof(int));
			b->IRQStat=0;
			break;
		case LAPCI_IOCTL_START_DMA0: /* Start DMA 0 */
			if(copy_from_user(&dp, (void*) param, sizeof(DMAParameters))) return -EFAULT;
			if(b->PLXrev==PCI_DEVICE_ID_PLX_9050)
				break;
			if(b->PLXrev==PCI_DEVICE_ID_PLX_9052)
				break;
			if(b->DMAbuff==NULL)
				break;
			StartDMA(b,0, &dp);
			break;
		case LAPCI_IOCTL_START_DMA1: /* Start DMA 1 */
			if(copy_from_user(&dp, (void*) param, sizeof(DMAParameters))) return -EFAULT;
			if(b->PLXrev==PCI_DEVICE_ID_PLX_9050)
				break;
			if(b->PLXrev==PCI_DEVICE_ID_PLX_9052)
				break;
			if(b->DMAbuff==NULL)
				break;
			StartDMA(b,1, &dp);
			break;
		case LAPCI_IOCTL_STOP_DMA0: /* Stop DMA 0 */
			if(b->PLXrev==PCI_DEVICE_ID_PLX_9050)
				break;
			if(b->PLXrev==PCI_DEVICE_ID_PLX_9052)
				break;
			StopDMA(b,0);
			break;
		case LAPCI_IOCTL_STOP_DMA1: /* Stop DMA 1 */
			if(b->PLXrev==PCI_DEVICE_ID_PLX_9050)
				break;
			if(b->PLXrev==PCI_DEVICE_ID_PLX_9052)
				break;
			StopDMA(b,1);
			break;
		case LAPCI_IOCTL_DMARDY: /* Get DMA status ready */
			if(copy_from_user(&dma, (void*) param, sizeof(int))) return -EFAULT;
			if(b->PLXrev==PCI_DEVICE_ID_PLX_9050)
				break;
			if(b->PLXrev==PCI_DEVICE_ID_PLX_9052)
				break;
			x=GetReadyDMA(b,dma);
			copy_to_user((void*) param, &x, sizeof(int));
			break;
		default:
			printk("\nLaPCI: unknown ioctl(%08X)\n", ioctl_num);
			return -EINVAL;
	}
	return 0;
}

/* Interrupt handler */
static void int_handler(int irq, void *dev_id, struct pt_regs *regs)
{
	BOARD_STRUCT *b;
	struct pci_dev *dev;
	int x;

	b = (BOARD_STRUCT*) dev_id;
	dev = b->pcidev;
	if(dev->irq != irq) 
		return;

	switch(b->PLXrev)
	{
		case PCI_DEVICE_ID_PLX_9050:
		case PCI_DEVICE_ID_PLX_9052:
			x = inl((b->IOBaseAdr[0]) + PLX9050_INTCSR);
			if(x & PLX9050_ICS_LI1S)
			{
				x = x & (~PLX9050_ICS_LI1E);
				outl(x, b->IOBaseAdr[0] + PLX9050_INTCSR);	/* Disable interrupt1 from PLX */
				b->IRQStat=x;
			}
			if(x & PLX9050_ICS_LI2S)
			{
				x = x & (~PLX9050_ICS_LI2E);
				outl(x, b->IOBaseAdr[0] + PLX9050_INTCSR);	/* Disable interrupt2 from PLX */
				b->IRQStat=x;
			}
			break;
		case PCI_DEVICE_ID_PLX_9054:
		case PCI_DEVICE_ID_PLX_9080:
			x = inl((b->IOBaseAdr[0]) + PLX9054_INTCSR);
			if(x & PLX9054_ICS_LIS)
			{
				x = x & (~PLX9054_ICS_LIE);
				outl(x, b->IOBaseAdr[0] + PLX9054_INTCSR);	/* Disable interrupt from PLX */
				b->IRQStat=x;
			}
			break;
	}
}

static struct file_operations fops =
{
	owner: THIS_MODULE,
	read: device_read,
	write: device_write,
	ioctl: device_ioctl,
	mmap: device_mmap,
	open: device_open,
	release: device_release,
}; 

/* Probing PCI device */
static int dev_probe(struct pci_dev *pcidev, const struct pci_device_id *pciid)
{
	BOARD_STRUCT *b;
	LaPCI_dev_info *di;
	unsigned int IOBaseAdr[6],IOSize[6];
	unsigned int MemBaseAdr[6],MemSize[6];
	unsigned int res_fl;
	int i,j;

	b=NULL;
	if(pciid==NULL)
		return -ENODEV;
	di=(LaPCI_dev_info*)(pciid->driver_data);
	if(di==NULL)
		return -ENODEV;

	for(i=0;i<6;i++)
	{
		IOBaseAdr[i]=0;
		IOSize[i]=0;
		MemBaseAdr[i]=0;
		MemSize[i]=0;
		res_fl=pci_resource_flags(pcidev,i+1);
		if(res_fl & IORESOURCE_IO)
		{
			IOBaseAdr[i]=pci_resource_start(pcidev,i+1);
			IOSize[i]=pci_resource_len(pcidev,i+1);
		}
		if(res_fl & IORESOURCE_MEM)
		{
			MemBaseAdr[i]=pci_resource_start(pcidev,i+1);
			MemSize[i]=pci_resource_len(pcidev,i+1);
		}
		
	}

	printk("\nLaPCI module probing %s\n",di->name);
/*	printk("VIDr=%x\n",pcidev->vendor);
	printk("Dev=%x\n",pcidev->device);
	printk("Subsys.VID=%x\n",pcidev->subsystem_vendor);
	printk("Subsys.Dev=%x\n",pcidev->subsystem_device);
*/

	b=kmalloc(sizeof(BOARD_STRUCT),GFP_KERNEL);
	if(!b)
		return -ENOMEM;
	memset(b,0,sizeof(BOARD_STRUCT));
	list_add_tail(&b->head,&dev_list);
	Boards++;
	pci_enable_device(pcidev);
	pci_set_drvdata(pcidev,b);

	b->pcidev = pcidev;
	b->subsys_id = di->subsys_id;
	strcpy(b->name, di->name);
	b->revision = di->revision;
	b->PLXrev = pcidev->device;
	b->slot = slot;
	b->DMAbuff = NULL;
	b->DMAsize=0;

	if(slot >= 0) 
		slot++;  /* Next board will use next slot */

	/*IO regions request */
	j=0;
	for(i=0;i<6;i++)
	{
		if(IOSize[i]>0)
		{
			if(!check_region(IOBaseAdr[i],IOSize[i]))
			{
				b->IOBaseAdr[j]=IOBaseAdr[i] & PCI_BASE_ADDRESS_IO_MASK;
				b->IOSize[j]=IOSize[i];
				request_region(b->IOBaseAdr[j],b->IOSize[j],device_name);
				printk("IO Base %08X size %08X\n",b->IOBaseAdr[j],b->IOSize[j]);
				j++;
			}
		}
	}

	/*Memory regions request */
	j=0;
	for(i=0;i<6;i++)
	{
		if(MemSize[i]>0)
		{
			if(!check_mem_region(MemBaseAdr[i],MemSize[i]))
			{
				b->MemBaseAdr[j]=MemBaseAdr[i] & PCI_BASE_ADDRESS_MEM_MASK;
				b->MemSize[j]=MemSize[i];
				request_mem_region(b->MemBaseAdr[j],b->MemSize[j],device_name);
				b->MemRemapAdr[j]=ioremap(b->MemBaseAdr[j],b->MemSize[j]);
				printk("Memory Base %08X size %08X\n",b->MemBaseAdr[j],b->MemSize[j]);
				j++;
			}
		}
	}

	request_irq(pcidev->irq, int_handler, SA_SHIRQ, device_name, (void*) b);
	return 0;
} 

/* Removing PCI device */ 
static void dev_remove(struct pci_dev *pcidev)
{
	BOARD_STRUCT *b;
	int i;

	b=(BOARD_STRUCT*)pci_get_drvdata(pcidev);
	plx_int_enable(b, 0);
	plx_reset(b);
	list_del(&b->head);
	Boards--;

	free_irq(pcidev->irq, (void*) b); 

	for(i=0;i<6;i++)
	{
		if((b->IOSize[i])>0)
			release_region(b->IOBaseAdr[i],b->IOSize[i]);
	}

	for(i=0;i<6;i++)
	{
		if((b->MemSize[i])>0)
		{
			release_mem_region(b->MemBaseAdr[i],b->MemSize[i]);
			iounmap(b->MemRemapAdr[i]);
			b->MemRemapAdr[i]=NULL;
		}
	}

	pci_set_drvdata(pcidev, NULL);
	if(b->DMAbuff!=NULL)
		free_dmabuf(b->DMAbuff,b->DMAsize);

	b->DMAbuff=NULL;
	b->DMAsize=0;
	kfree(b);
} 

static struct pci_driver LaPCI_driver =
{
	name: device_name,
	id_table: id_table,
	probe: dev_probe,
	remove: dev_remove
}; 

/* Driver initialization */ 
int init_LaPCI() 
{ 
	if(!pci_present()) 
		return -ENODEV;
	if (register_chrdev(Major, device_name, &fops) < 0)	/*    */
	{ 
		printk("\nLaPCI: Register failed\n"); 
	} 
	return pci_module_init(&LaPCI_driver);  
}

/* Unloading */  
void cleanup_LaPCI() 
{ 
	if (unregister_chrdev(Major, device_name) < 0)		/*    */
	{ 
		printk("LaPCI: unregister device failed\n"); 
	} 

	printk("LaPCI is dead...\n");
	pci_unregister_driver(&LaPCI_driver); 
	return; 
}
 
module_init(init_LaPCI); 
module_exit(cleanup_LaPCI);
